@@ -18,7 +18,7 @@
 
 """Provides functions and classes for getting/sending files to destination"""
 
-import os, types, ftplib
+import os, types, ftplib, tempfile
 import log, path, dup_temp, file_naming
 
 class BackendException(Exception): pass
@@ -137,13 +137,13 @@ class Backend:
 
 	def run_command(self, commandline):
 		"""Run given commandline with logging and error detection"""
-		log.Log("Running '%s'" % commandline, 4)
+		log.Log("Running '%s'" % commandline, 5)
 		if os.system(commandline):
 			raise BackendException("Error running '%s'" % commandline)
 
 	def popen(self, commandline):
 		"""Run command and return stdout results"""
-		log.Log("Reading results of '%s'" % commandline, 4)
+		log.Log("Reading results of '%s'" % commandline, 5)
 		fout = os.popen(commandline)
 		results = fout.read()
 		if fout.close():
@@ -343,14 +343,14 @@ class ftpBackend(Backend):
 		"""Transfer source_path to remote_filename"""
 		if not remote_filename: remote_filename = source_path.get_filename()
 		source_file = source_path.open("rb")
-		log.Log("Saving %s on FTP server" % (remote_filename,), 4)
+		log.Log("Saving %s on FTP server" % (remote_filename,), 5)
 		self.error_wrap('storbinary', "STOR "+remote_filename, source_file)
 		assert not source_file.close()
 
 	def get(self, remote_filename, local_path):
 		"""Get remote filename, saving it to local_path"""
 		target_file = local_path.open("wb")
-		log.Log("Retrieving %s from FTP server" % (remote_filename,), 4)
+		log.Log("Retrieving %s from FTP server" % (remote_filename,), 5)
 		self.error_wrap('retrbinary', "RETR "+remote_filename,
 						target_file.write)
 		assert not target_file.close()
@@ -358,13 +358,13 @@ class ftpBackend(Backend):
 
 	def list(self):
 		"""List files in directory"""
-		log.Log("Listing files on FTP server", 4)
+		log.Log("Listing files on FTP server", 5)
 		return self.error_wrap('nlst')
 
 	def delete(self, filename_list):
 		"""Delete files in filename_list"""
 		for filename in filename_list:
-			log.Log("Deleting %s from FTP server" % (filename,), 4)
+			log.Log("Deleting %s from FTP server" % (filename,), 5)
 			self.error_wrap('delete', filename)
 
 	def close(self):
@@ -372,11 +372,80 @@ class ftpBackend(Backend):
 		self.error_wrap('quit')
 
 
-# Dictionary relating protocol strings to tuples (backend_object,
-# separate_host).  If separate_host is true, get_backend() above will
-# parse the url further to try to extract a hostname, protocol, etc.
+class rsyncBackend(Backend):
+	"""Connect to remote store using rsync
+
+	rsync backend contributed by Sebastian Wilhelmi <seppi@seppi.de>
+
+	"""
+	def __init__(self, parsed_url):
+		"""rsyncBackend initializer"""
+		self.url_string = parsed_url.url_string
+		if self.url_string[-1] != '/':
+			self.url_string += '/'
+
+	def put(self, source_path, remote_filename = None):
+		"""Use rsync to copy source_dir/filename to remote computer"""
+		if not remote_filename: remote_filename = source_path.get_filename()
+		remote_path = os.path.join (self.url_string, remote_filename)
+		commandline = "rsync %s %s" % (source_path.name, remote_path)
+		self.run_command(commandline)
+
+	def get(self, remote_filename, local_path):
+		"""Use rsync to get a remote file"""
+		remote_path = os.path.join (self.url_string, remote_filename)
+		commandline = "rsync %s %s" % (remote_path, local_path.name)
+		self.run_command(commandline)
+		local_path.setdata()
+		if not local_path.exists():
+			raise BackendException("File %s not found" % local_path.name)
+		
+	def list(self):
+		"""List files"""
+		def split (str):
+			line = str.split ()
+			if len (line) > 4 and line[4] != '.':
+				return line[4]
+			else:
+				return None
+		commandline = "rsync %s" % self.url_string
+		return filter (lambda x: x, map (split, self.popen(commandline).split('\n')))
+
+	def delete(self, filename_list):
+		"""Delete files."""
+		delete_list = filename_list
+		dont_delete_list = []
+		for file in self.list ():
+			if file in delete_list:
+				delete_list.remove (file)
+			else:
+				dont_delete_list.append (file)
+		if len (delete_list) > 0:
+			raise BackendException("Files %s not found" % str (delete_list))
+
+		dir = tempfile.mktemp ()
+		exclude_name = tempfile.mktemp ()
+		exclude = open (exclude_name, 'w')
+		to_delete = [exclude_name]
+		os.mkdir (dir)
+		for file in dont_delete_list:
+				path = os.path.join (dir, file)
+				to_delete.append (path)
+				f = open (path, 'w')
+				f.close ()
+				print >>exclude, file
+		exclude.close ()
+		commandline = ("rsync --recursive --delete --exclude-from=%s %s/ %s" %
+			       (exclude_name, dir, self.url_string))
+		self.run_command(commandline)
+		for file in to_delete:
+			os.unlink (file)
+		os.rmdir (dir)
+		
+# Dictionary relating protocol strings to backend_object classes.
 protocol_class_dict = {"scp": scpBackend,
 					   "ssh": scpBackend,
 					   "file": LocalBackend,
-					   "ftp": ftpBackend}
+					   "ftp": ftpBackend,
+					   "rsync": rsyncBackend}
 
