@@ -92,8 +92,9 @@ class BackupSet:
 
 	def delete(self):
 		"""Remove all files in set"""
-		self.backend.delete(self.remote_manfest_name)
-		for filename in self.volume_name_dict: self.backend.delete(filename)
+		l = self.get_filenames()
+		l.reverse() # delete starting with manifest
+		self.backend.delete(l)
 
 	def __str__(self):
 		"""For now just list files in set"""
@@ -101,7 +102,7 @@ class BackupSet:
 		if self.remote_manifest_name:
 			filelist.append(self.remote_manifest_name)
 		filelist.extend(self.volume_name_dict.values())
-		return "\n".join(filelist)
+		return "[%s]" % ", ".join(filelist)
 
 	def get_timestr(self):
 		"""Return time string suitable for log statements"""
@@ -140,6 +141,23 @@ remote backup set or the local archive directory has been corrupted.""")
 		if self.local_manifest_path: return self.get_local_manifest()
 		else: return self.get_remote_manifest()
 
+	def get_filenames(self):
+		"""Return sorted list of (remote) filenames of files in set"""
+		assert self.info_set
+		volume_num_list = self.volume_name_dict.keys()
+		volume_num_list.sort()
+		volume_filenames = map(lambda x: self.volume_name_dict[x],
+							   volume_num_list)
+		if self.remote_manifest_name:
+			return [self.remote_manifest_name] + volume_filenames
+		else: return volume_filenames
+		
+	def get_time(self):
+		"""Return time if full backup, or end_time if incremental"""
+		if self.time: return self.time
+		if self.end_time: return self.end_time
+		assert 0, "Neither self.time nor self.end_time set"
+
 
 class BackupChain:
 	"""BackupChain - a number of linked BackupSets
@@ -175,6 +193,7 @@ class BackupChain:
 		"""Delete all sets in chain, in reverse order"""
 		for i in range(len(self.incset_list)-1, -1, -1):
 			self.incset_list[i].delete()
+		if self.fullset: self.fullset.delete()
 
 	def get_sets_at_time(self, time):
 		"""Return a list of sets in chain earlier or equal to time"""
@@ -186,6 +205,20 @@ class BackupChain:
 		if self.incset_list: return self.incset_list[-1]
 		else: return self.fullset
 
+	def __str__(self):
+		"""Return string representation, for testing purposes"""
+		incset_str = "{%s}" % ", ".join(map(str, self.incset_list))
+		return ("BackupChain: Fullset: %s\n"
+				"             Incsetlist: %s\n"
+				"             Start Time: %s\n"
+				"             End Time: %s" % (self.fullset, incset_str,
+											   self.start_time, self.end_time))
+
+	def get_all_sets(self):
+		"""Return list of all backup sets in chain"""
+		if self.fullset: return [self.fullset] + self.incset_list
+		else: return self.incset_list
+
 
 class SignatureChain:
 	"""A number of linked signatures
@@ -262,6 +295,13 @@ class SignatureChain:
 			inclist_copy.append(self.fullsig)
 			self.backend.delete(inclist_copy)
 
+	def get_filenames(self):
+		"""Return ordered list of filenames in set"""
+		if self.fullsig: l = [self.fullsig]
+		else: l = []
+		l.extend(self.inclist)
+		return l
+
 
 class CollectionsStatus:
 	"""Hold information about available chains and sets"""
@@ -297,7 +337,10 @@ class CollectionsStatus:
 			 "Orphaned sig names: %s" % (self.orphaned_sig_names,),
 			 "Orphaned backup sets: %s" % (self.orphaned_backup_sets,),
 			 "Incomplete backup sets: %s" % (self.incomplete_backup_sets,)]
-		return "\n".join(l)
+		part1 = "\n".join(l)
+		part2 = "\n----------------Backup Chains----------------------\n"
+		part3 = "\n------\n".join(map(str, self.all_backup_chains))
+		return part1+part2+part3
 
 	def set_values(self, sig_chain_warning = 1):
 		"""Set values from archive_dir and backend.
@@ -311,8 +354,9 @@ class CollectionsStatus:
 		self.values_set = 1
 		backend_filename_list = self.backend.list()
 
-		backup_chains, self.orphaned_backup_sets, self.incomplete_backup_set=\
-					   self.get_backup_chains(backend_filename_list)
+		(backup_chains, self.orphaned_backup_sets,
+		         self.incomplete_backup_sets) = \
+				 self.get_backup_chains(backend_filename_list)
 		backup_chains = self.get_sorted_chains(backup_chains)
 		self.all_backup_chains = backup_chains
 
@@ -368,7 +412,8 @@ class CollectionsStatus:
 					"from aborted session", 2)
 		if self.orphaned_backup_sets:
 			log.Log("Warning, found the following orphaned backup files:\n"
-					+ "\n".join(map(lambda x: str(x), orphaned_sets)), 2)
+					+ "\n".join(map(lambda x: str(x),
+									self.orphaned_backup_sets)), 2)
 
 	def get_backup_chains(self, filename_list):
 		"""Split given filename_list into chains
@@ -411,7 +456,7 @@ class CollectionsStatus:
 		"""Sort set list by end time, return (sorted list, incomplete)"""
 		time_set_pairs, incomplete_sets = [], []
 		for set in set_list:
-			if not set.is_complete: incomplete_sets.append(set)
+			if not set.is_complete(): incomplete_sets.append(set)
 			elif set.type == "full": time_set_pairs.append((set.time, set))
 			else: time_set_pairs.append((set.end_time, set))
 		time_set_pairs.sort()
@@ -509,3 +554,63 @@ class CollectionsStatus:
 	def cleanup_signatures(self):
 		"""Delete unnecessary older signatures"""
 		map(SignatureChain.delete, self.other_sig_chains)
+
+	def get_extraneous(self):
+		"""Return list of the names of extraneous duplicity files
+
+		A duplicity file is considered extraneous if it is
+		recognizable as a duplicity file, but isn't part of some
+		complete backup set, or current signature chain.
+
+		"""
+		assert self.values_set
+		filenames = []
+		ext_containers = (self.other_sig_chains, self.orphaned_backup_sets,
+						  self.incomplete_backup_sets)
+		for set_or_chain_list in ext_containers:
+			for set_or_chain in set_or_chain_list:
+				filenames.extend(set_or_chain.get_filenames())
+		filenames.extend(self.orphaned_sig_names)
+		return filenames
+
+	def sort_sets(self, setlist):
+		"""Return new list containing same elems of setlist, sorted by time"""
+		pairs = map(lambda s: (s.get_time(), s), setlist)
+		pairs.sort()
+		return map(lambda p: p[1], pairs)
+
+	def get_chains_older_than(self, t):
+		"""Return a list of chains older than time t"""
+		assert self.values_set
+		return filter(lambda c: c.end_time < t, self.all_backup_chains)
+
+	def get_older_than(self, t):
+		"""Returns a list of backup sets older than the given time t
+
+		All of the times will be associated with an intact chain.
+		Furthermore, none of the times will be of a set which a newer
+		set may depend on.  For instance, if set A is a full set older
+		than t, and set B is an incremental based on A which is newer
+		than tt, then the time of set A will not be returned.
+
+		"""
+		old_sets = []
+		for chain in self.get_chains_older_than(t):
+			old_sets.extend(chain.get_all_sets())
+		return self.sort_sets(old_sets)
+
+	def get_older_than_required(self, t):
+		"""Returns list of old backup sets required by new sets
+
+		This function is similar to the previous one, but it only
+		returns the times of sets which are old but part of the chains
+		where the newer end of the chain is newer than t.
+
+		"""
+		assert self.values_set
+		new_chains = filter(lambda c: c.end_time >= t, self.all_backup_chains)
+		result_sets = []
+		for chain in new_chains:
+			old_sets = filter(lambda s: s.get_time() < t, chain.get_all_sets())
+			result_sets.extend(old_sets)
+		return self.sort_sets(result_sets)
